// Copyright 2020, Ryan Wendland, usb64
// SPDX-License-Identifier: MIT

#ifndef _N64_CONTROLLER_h
#define _N64_CONTROLLER_h

#ifdef __cplusplus
extern "C" {
#endif

#include <stdint.h>
#include "n64_controllerpak.h"
#include "n64_rumblepak.h"
#include "n64_settings.h"
#include "n64_virtualpak.h"
#include "n64_transferpak_gbcarts.h"

typedef struct  __attribute__((packed))
{
    uint16_t dButtons;
    int8_t x_axis;
    int8_t y_axis;
} n64_buttonmap;

typedef struct __attribute__((packed))
{
    uint8_t led_state;
    uint16_t buttons[3];
    uint8_t flags;
} n64_randnet_kb;

typedef enum
{
    N64_CONTROLLER,
    N64_MOUSE,
    N64_RANDNET
} n64_input_type;

typedef enum
{
    PERI_NONE,
    PERI_RPAK,
    PERI_CPAK,
    PERI_TPAK
} n64_peri_type;

typedef struct
{
    uint32_t id;                      //Controller ID
    int32_t current_bit;              //The current bit to being received in
    uint32_t current_byte;            //The current byte being received in
    uint8_t data_buffer[50];          //Controller main tx and rx buffer
    uint32_t peri_access;             //Peripheral flag is set when a peripheral is being accessed
    uint32_t crc_error;               //Set if the 2 byte address has a CRC error.
    n64_input_type type;              //Store the type of input device. Controller, Mouse. Randnet etc.
    n64_buttonmap b_state;            //N64 controller button and analog stick map
    n64_randnet_kb kb_state;          //Randnet keyboard object
    n64_peri_type current_peripheral; //Peripheral flag, PERI_NONE, PERI_RPAK, PERI_CPAK, PERI_TPAK
    n64_peri_type next_peripheral;    //What Peripheral to change to next after timer
    n64_transferpak *tpak;            //Pointer to installed transferpak
    n64_rumblepak *rpak;              //Pointer to installed rumblepak
    n64_controllerpak *cpak;          //Pointer to installed controllerpak
                                      //
    uint8_t  connected;               //Flag is set when this controller is connected
    uint32_t interrupt_attached;      //Flag is set when this controller is connected to an ext int.
    uint32_t bus_idle_timer_clks;     //Timer counter for bus idle timing
    usb64_pin_t  pin;                 //What pin is this controller connected to
} n64_input_dev_t;

typedef struct
{
    uint32_t crc_hi;
    uint32_t crc_low;
    uint8_t media_format;
    uint8_t country_code;
    const char *name;
} n64_game_t;

typedef struct 
{
    uint32_t crc_hi;
    uint32_t crc_low;
    const char *name;
} n64_game_crc_lookup_t;

//N64 JOYBUS
#define N64_IDENTIFY 0x00
#define N64_CONTROLLER_STATUS 0x01
#define N64_PERI_READ 0x02
#define N64_PERI_WRITE 0x03
#define N64_CONTROLLER_RESET 0xFF
#define N64_RANDNET_REQ 0x13
#define N64_ED64_GAMEID 0x1D
#define N64_COMMAND_POS 0
#define N64_ADDRESS_MSB_POS 1
#define N64_ADDRESS_LSB_POS 2
#define N64_DATA_POS 3
#define N64_CRC_POS 35
#define N64_MAX_POS 36

//N64 Controller
#define N64_A (1UL << 7)
#define N64_B (1UL << 6)
#define N64_Z (1UL << 5)
#define N64_ST (1UL << 4)
#define N64_DU (1UL << 3)
#define N64_DD (1UL << 2)
#define N64_DL (1UL << 1)
#define N64_DR (1UL << 0)
#define N64_RES (1UL << 15)
#define NOTUSED (1UL << 14)
#define N64_LB (1UL << 13)
#define N64_RB (1UL << 12)
#define N64_CU (1UL << 11)
#define N64_CD (1UL << 10)
#define N64_CL (1UL << 9)
#define N64_CR (1UL << 8)

//RANDNET
#define RANDNET_LED_POS     1
#define RANDNET_BTN_POS     2
#define RANDNET_MAX_BUTTONS 3
#define RANDNET_LED_NUMLOCK            (1 << 0)
#define RANDNET_LED_CAPSLOCK           (1 << 1)
#define RANDNET_LED_POWER              (1 << 2)
#define RANDNET_FLAG_HOME_KEY          (1 << 0)
#define RANDNET_FLAG_EXCESS_BUTTONS    (1 << 4)

//Mempak
#define CPAK_SIZE 32768
#define MAX_CPAKS MAX_CONTROLLERS
#define VIRTUAL_PAK MAX_CPAKS

void n64_subsystem_init(n64_input_dev_t *in_dev);
void n64_controller_hande_new_edge(n64_input_dev_t *cont);
//Only works with compatible ED64 flashcarts
const char *n64_get_current_game();

static const FLASHMEM n64_game_crc_lookup_t game_crc_lookup[] = {
{0x3B941695, 0xF90A5EEB,
"007 - The World is Not Enough (E) (M3)"},
{0x033F4C13, 0x319EE7A7,
"007 - The World is Not Enough (U)"},
{0x58FD3F25, 0xD92EAA8D,
"1080 Snowboarding (E) [!]"},
{0x1FBAF161, 0x2C1C54F1,
"1080 Snowboarding (JU) [!]"},
{0xABA51D09, 0xC668BAD9,
"40 Winks (E) (M3) (Beta)"},
{0xB98BA456, 0x5B2B76AF,
"64 de Hakken!! Tamagotchi Minna de Tamagotchi World (J)"},
{0x9C961069, 0xF5EA488D,
"64 Oozumou (J)"},
{0x85C18B16, 0xDF9622AF,
"64 Oozumou 2 (J)"},
{0x7A6081FC, 0xFF8F7A78,
"64 Trump Collection - Alice no Wakuwaku Trump World (J) [!]"},
{0x8F12C096, 0x45DC17E1,
"A Bug's Life (E) [!]"},
{0x2B38AEC0, 0x6350B810,
"A Bug's Life (F) [!]"},
{0xDFF227D9, 0x0D4D8169,
"A Bug's Life (G) [!]"},
{0xF63B89CE, 0x4582D57D,
"A Bug's Life (I)"},
{0x82DC04FD, 0xCF2D82F4,
"A Bug's Life (U) [!]"},
{0x62F6BE95, 0xF102D6D6,
"Aerofighter's Assault (E) [!]"},
{0x1B598BF1, 0xECA29B45,
"Aerofighter's Assault (U) [!]"},
{0xD83045C8, 0xF29D3A36,
"AeroGauge (E) [!]"},
{0xB00903C9, 0x3916C146,
"AeroGauge - Kiosk (J) (V1.0)"},
{0x80F41131, 0x384645F6,
"AeroGauge (J) (V1.1) [!]"},
{0xAEBE463E, 0xCC71464B,
"AeroGauge (U) [!]"},
{0x8CC182A6, 0xC2D0CAB0,
"AI Shogi 3 (J) [!]"},
{0x2DC4FFCC, 0xC8FF5A21,
"Aidyn Chronicles - The First Mage (E) [!]"},
{0xE6A95A4F, 0xBAD2EA23,
"Aidyn Chronicles - The First Mage (U) [!] (V1.0)"},
{0x112051D2, 0x68BEF8AC,
"Aidyn Chronicles - The First Mage (U) [!] (V1.1)"},
{0x27C425D0, 0x8C2D99C1,
"Airboarder 64 (E) [!]"},
{0x6C45B60C, 0xDCE50E30,
"Airboarder 64 (J) [!]"},
{0xB6951A94, 0x63C849AF,
"Akumajou Dracula Mokushiroku - Real Action Adventure (J) [!]"},
{0xA5533106, 0xB9F25E5B,
"Akumajou Dracula Mokushiroku Gaiden-Legend of Cornell (J) [!]"},
{0xDFD784AD, 0xAE426603,
"All Star Tennis '99 (E) [!]"},
{0xE185E291, 0x4E50766D,
"All Star Tennis '99 (U) [!]"},
{0x67D20729, 0xF696774C,
"All Star! Dairantou Smash Brothers (J) [!]"},
{0xA19F8089, 0x77884B51,
"All-Star Baseball 2000 (E) [!]"},
{0x5E547A4D, 0x90E60795,
"All-Star Baseball 2000 (U) [!]"},
{0x5446C6EF, 0xE18E47BB,
"All-Star Baseball 2001 (U) [!]"},
{0xD9EDD54D, 0x6BB8E274,
"All-Star Baseball 99 (E) [!]"},
{0xC43E23A7, 0x40B1681A,
"All-Star Baseball 99 (U) [!]"},
{0x3CC77150, 0x21CDB987,
"Armorines - Project S.W.A.R.M. (E) [!]"},
{0xC0F6DB17, 0x80E0D532,
"Armorines - Project S.W.A.R.M. (G) [!]"},
{0x1FB5D932, 0x3BA9481B,
"Armorines - Project S.W.A.R.M. (U) [!]"},
{0x4C52BBB2, 0xCEAB0F6B,
"Army Men - Air Combat (U) [!]"},
{0xB210DF19, 0x98B58D1A,
"Army Men - Sarge's Heroes (E) [!]"},
{0x862C0657, 0x8DFD896D,
"Army Men - Sarge's Heroes (U) [!]"},
{0xB20F73B6, 0x2975FC34,
"Army Men - Sarge's Heroes 2 (U) [!]"},
{0xD1F7D8AB, 0x293B0446,
"Asteroids Hyper 64 (U) [!]"},
{0xFC7797BF, 0x4A95E83C,
"Automobili Lamborghini (E) [!]"},
{0x41B25DC4, 0x1B726786,
"Automobili Lamborghini (U) [!]"},
{0xE340A49C, 0x74318D41,
"Baku Bomberman (J) [!]"},
{0xE73C7C4F, 0xAF93B838,
"Baku Bomberman 2 (J) [!]"},
{0xDF98B95D, 0x58840978,
"Bakuretsu Muteki Bangaioh (J) [!]"},
{0x88CF980A, 0x8ED52EB5,
"Bakushou Jinsei 64 - Mezase! Resort Ou (J) [!]"},
{0x5168D520, 0xCA5FCD0D,
"Banjo to Kazooie no Dai Bouken (J) [!]"},
{0x514B6900, 0xB4B19881,
"Banjo to Kazooie no Dai Bouken 2 (J) [!]"},
{0x733FCCB1, 0x444892F9,
"Banjo-Kazooie (E) [!]"},
{0xA4BF9306, 0xBF0CDFD1,
"Banjo-Kazooie (U) [!]"},
{0x155B7CDF, 0xF0DA7325,
"Banjo-Tooie (A) [!]"},
{0xC9176D39, 0xEA4779D1,
"Banjo-Tooie (E) [!]"},
{0xC2E9AA9A, 0x475D70AA,
"Banjo-Tooie (U) [!]"},
{0xB088FBB4, 0x441E4B1D,
"Bass Hunter 64 (E) [!]"},
{0xD76333AC, 0x0CB6219D,
"Bass Rush - Ecogear Powerworm Championship (J)"},
{0xD137A2CA, 0x62B65053,
"Bass Tsuri No. 1 - Shigesato Itoi's Bass Fishing (J) [!]"},
{0xBCFACCAA, 0xB814D8EF,
"Bassmasters 2000 (U) [!]"},
{0x259F7F84, 0x7C9EED26,
"Batman of the Future - Return of the Joker (E) [!]"},
{0x204489C1, 0x1286CF2B,
"Batman Beyond - Return of the Joker (U) [!]"},
{0x0CAD17E6, 0x71A5B797,
"BattleTanx - Global Assault (E) [!]"},
{0x75A4E247, 0x6008963D,
"BattleTanx - Global Assault (U) [!]"},
{0x6AA4DDE7, 0xE3E2F4E7,
"BattleTanx (U) [!]"},
{0x55D4C4CE, 0x7753C78A,
"Battlezone - Rise of the Black Dogs (U) [!]"},
{0x91691C3D, 0xF4AC5B4D,
"Beast Wars Metals (J) [!]"},
{0x4D79D316, 0xE8501B33,
"Beast Wars Transmetal (U) [!]"},
{0xA1B64A61, 0xD014940B,
"Beetle Adventure Racing! (E) [!]"},
{0x9C7318D2, 0x24AE0DC1,
"Beetle Adventure Racing! (J) [!]"},
{0xEDF419A8, 0xBF1904CC,
"Beetle Adventure Racing! (U) [!]"},
{0x08FFA4B7, 0x01F453B6,
"Big Mountain 2000 (U) [!]"},
{0xAB7C101D, 0xEC58C8B0,
"Bio F.R.E.A.K.S. (E) [!]"},
{0x08123595, 0x0510F1DE,
"Bio F.R.E.A.K.S. (U) [!]"},
{0x7EAE2488, 0x9D40A35A,
"Bio Hazard 2 (J) [!]"},
{0x7C64E6DB, 0x55B924DB,
"Blast Corps (E) [!]"},
{0x7C647C25, 0xD9D901E6,
"Blast Corps (U) (V1.0) [!]"},
{0x7C647E65, 0x1948D305,
"Blast Corps (U) (V1.1) [!]"},
{0x65234451, 0xEBD3346F,
"Blast Dozer (J) [!]"},
{0xD571C883, 0x822D3FCF,
"Blues Brothers 2000 (E) [!]"},
{0x7CD08B12, 0x1153FF89,
"Blues Brothers 2000 (U) [!]"},
{0x0B58B8CD, 0xB7B291D2,
"Body Harvest (E) [!]"},
{0x5326696F, 0xFE9A99C3,
"Body Harvest (U) [!]"},
{0xB3D451C6, 0xE1CB58E2,
"Bokujo Monogatari 2 (J) (V1.0) [!]"},
{0x736657F6, 0x3C88A702,
"Bokujo Monogatari 2 (J) (V1.1)"},
{0x7365D8F8, 0x9ED9326F,
"Bokujo Monogatari 2 (J) (V1.2)"},
{0xDF6FF0F4, 0x29D14238,
"Bomberman 64 - Arcade Edition (J) [!]"},
{0x237E73B4, 0xD63B6B37,
"Bomberman 64 - The Second Attack! (U) [!]"},
{0x5A160336, 0xBC7B37B0,
"Bomberman 64 (E) [!]"},
{0xF568D51E, 0x7E49BA1E,
"Bomberman 64 (U) [!]"},
{0xD85C4E29, 0x88E276AF,
"Bomberman Hero (E) [!]"},
{0x67FF12CC, 0x76BF0212,
"Bomberman Hero (J) [!]"},
{0x4446FDD6, 0xE3788208,
"Bomberman Hero (U) [!]"},
{0xD72FD14D, 0x1FED32C4,
"Bottom of the 9th (U) [!]"},
{0x1E22CF2E, 0x42AAC813,
"Brunswick Circuit Pro Bowling (U) [!]"},
{0xD5B2339C, 0xCABCCAED,
"Buck Bumble (E) [!]"},
{0xD7C762B6, 0xF83D9642,
"Buck Bumble (J) [!]"},
{0x85AE781A, 0xC756F05D,
"Buck Bumble (U) [!]"},
{0xCEDCDE1E, 0x513A0502,
"Bust-A-Move 2 - Arcade Edition (E) [!]"},
{0x8A86F073, 0xCD45E54B,
"Bust-A-Move 2 - Arcade Edition (U) [!]"},
{0xE328B4FA, 0x004A28E1,
"Bust-A-Move 3 DX (E) [!]"},
{0x4222D89F, 0xAFE0B637,
"Bust-A-Move '99 (U) [!]"},
{0xAC16400E, 0xCF5D071A,
"California Speed (U) [!]"},
{0x580162EC, 0xE3108BF1,
"Carmageddon 64 (G) [!]"},
{0xE48E01F5, 0xE6E51F9B,
"Carmageddon 64 (E) [!]"},
{0xF00F2D4E, 0x340FAAF4,
"Carmageddon 64 (U) [!]"},
{0xA2C54BE7, 0x6719CBB2,
"Castlevania - Legacy of Darkness (E) [!]"},
{0x1CC06338, 0x87388926,
"Castlevania - Legacy of Darkness (U) [!]"},
{0x64F1B7CA, 0x71A23755,
"Castlevania (E) [!]"},
{0xF35D5F95, 0x8AFE3D69,
"Castlevania (U) (V1.0) [!]"},
{0x4BCDFF47, 0xAAA3AF8F,
"Castlevania (U) (V1.2) [!]"},
{0xDCCF2134, 0x9DD63578,
"Centre Court Tennis (E) [!]"},
{0xB9AF8CC6, 0xDEC9F19F,
"Chameleon Twist (E) [!]"},
{0xA4F2F521, 0xF0EB168E,
"Chameleon Twist (J) [!]"},
{0x6420535A, 0x50028062,
"Chameleon Twist (U) [!]"},
{0x07A69D01, 0x9A7D41A1,
"Chameleon Twist 2 (E) [!]"},
{0x0549765A, 0x93B9D042,
"Chameleon Twist 2 (J) [!]"},
{0xCD538CE4, 0x618AFCF9,
"Chameleon Twist 2 (U) [!]"},
{0xFB3C48D0, 0x8D28F69F,
"Charlie Blast's Territory (E) [!]"},
{0x1E0E96E8, 0x4E28826B,
"Charlie Blast's Territory (U) [!]"},
{0x2E359339, 0x3FA5EDA6,
"Chopper Attack (E) [!]"},
{0x214CAD94, 0xBE1A3B24,
"Chopper Attack (U) [!]"},
{0x2BCCF9C4, 0x403D9F6F,
"Choro Q 64 (J) [!]"},
{0x26CD0F54, 0x53EBEFE0,
"Choro Q 64 II (J) [!]"},
{0x8ACE6683, 0x3FBA426E,
"Chou Kuukan Night Pro Yakyuu King (J)"},
{0x3A180FF4, 0x5C8E8AF7,
"Chou-Kuukan Night Pro Yakyuu King 2 (J)"},
{0xA7941528, 0x61F1199D,
"Chou Snowbow Kids (J) [!]"},
{0xF8009DB0, 0x6B291823,
"City-Tour GP-All Japan Grand Touring Car Championship (J) [!]"},
{0xFA5A3DFF, 0xB4C9CDB9,
"Clay Fighter - Sculptor's Cut (U) [!]"},
{0x2B6FA7C0, 0x09A71225,
"Clay Fighter 63 1-3 (U) (beta)"},
{0x8E9692B3, 0x4264BB2A,
"Clay Fighter 63 1-3 (E) [!]"},
{0xF03C24CA, 0xC5237BCC,
"Clay Fighter 63 1-3 (U) [!]"},
{0xAE5B9465, 0xC54D6576,
"Command & Conquer (E) [!]"},
{0xB5025BAD, 0xD32675FD,
"Command & Conquer (G) [!]"},
{0x95286EB4, 0xB76AD58F,
"Command & Conquer (U) [!]"},
{0x373F5889, 0x9A6CA80A,
"Conker's Bad Fur Day (E) [!]"},
{0x30C7AC50, 0x7704072D,
"Conker's Bad Fur Day (U) [!]"},
{0x46A3F7AF, 0x0F7591D0,
"Cruis'n Exotica (U) [!]"},
{0x503EA760, 0xE1300E96,
"Cruis'n USA (E) [!]"},
{0xFF2F2FB4, 0xD161149A,
"Cruis'n USA (U) (V1.0) [!]"},
{0x5306CF45, 0xCBC49250,
"Cruis'n USA (U) (V1.1) [!]"},
{0xB3402554, 0x7340C004,
"Cruis'n USA (U) (V1.2) [!]"},
{0x83F3931E, 0xCB72223D,
"Cruis'n World (E) [!]"},
{0xDFE61153, 0xD76118E6,
"Cruis'n World (U) [!]"},
{0x83CB0B87, 0x7E325457,
"Custom Robo (J) [!]"},
{0x079501B9, 0xAB0232AB,
"Custom Robo V2 (J) [!]"},
{0xD1A78A07, 0x52A3DD3E,
"CyberTiger (E) [!]"},
{0xE8FC8EA1, 0x9F738391,
"CyberTiger (U) [!]"},
{0x7188F445, 0x84410A68,
"Dance Dance Revolution - Disney Dancing Museum (J) [!]"},
{0x7ED67CD4, 0xB4415E6D,
"Dark Rift (E) [!]"},
{0xA4A52B58, 0x23759841,
"Dark Rift (U) [!]"},
{0xF5363349, 0xDBF9D21B,
"Deadly Arts (U) [!]"},
{0x3F66A9D9, 0x9BCB5B00,
"Defi au Tetris Magique (Magical Tetris Challenge) (F) [!]"},
{0x17C54A61, 0x4A83F2E7,
"Densha de GO! 64 (J) [!]"},
{0xA5F667E1, 0xDA1FBD1F,
"Derby Stallion (J) [!]"},
{0x630AA37D, 0x896BD7DB,
"Destruction Derby 64 (E) [!]"},
{0xDEE584A2, 0x0F161187,
"Destruction Derby 64 (U) [!]"},
{0x76712159, 0x35666812,
"Dexanoid R1 by Protest Design (PD) [!]"},
{0x8979169C, 0xF189F6A0,
"Dezaemon 3D (J) [!]"},
{0xFD73F775, 0x9724755A,
"Diddy Kong Racing (E) (V1.0) [!]"},
{0x596E145B, 0xF7D9879F,
"Diddy Kong Racing (E) (V1.1) [!]"},
{0xF389A35A, 0x17785562,
"Diddy Kong Racing (J) [!]"},
{0x53D440E7, 0x7519B011,
"Diddy Kong Racing (U) (V1.0) [!]"},
{0xE402430D, 0xD2FCFC9D,
"Diddy Kong Racing (U) (V1.1) [!]"},
{0x906C3F77, 0xCE495EA1,
"Dinosaur Planet (U) (Beta) (Unreleased)"},
{0xC16C421B, 0xA21580F7,
"Disney's Donald Duck - Goin' Quackers (U) [!]"},
{0xD614E5BF, 0xA76DBCC1,
"Disney's Tarzan (E) [!]"},
{0x001A3BD0, 0xAFB3DE1A,
"Disney's Tarzan (F) [!]"},
{0x4C261323, 0x4F295E1A,
"Disney's Tarzan (G) [!]"},
{0xCBFE69C7, 0xF2C0AB2A,
"Disney's Tarzan (U) [!]"},
{0xBD8E206D, 0x98C35E1C,
"Dobutsu no Mori (J) [!]"},
{0x3DF17480, 0x193DED5A,
"Donald Duck - Quack Attack (E) [!]"},
{0x0DD4ABAB, 0xB5A2A91E,
"Donkey Kong 64 - Kiosk (U) [!]"},
{0x11936D8C, 0x6F2C4B43,
"Donkey Kong 64 (E) [!]"},
{0x053C89A7, 0xA5064302,
"Donkey Kong 64 (J) [!]"},
{0xEC58EABF, 0xAD7C7169,
"Donkey Kong 64 (U) [!]"},
{0x2C739EAC, 0x9EF77726,
"Doom 64 (E) [!]"},
{0x7AA65B36, 0xFDCEE5AD,
"Doom 64 (J) [!]"},
{0xA83E101A, 0xE937B69D,
"Doom 64 (U) (V1.0) [!]"},
{0x423E96F4, 0xCE88F05B,
"Doom 64 (U) (v1.1)"},
{0xBFF7B1C2, 0xAEBF148E,
"Doraemon - Mittsu no Seireiseki (J) [!]"},
{0xB6306E99, 0xB63ED2B2,
"Doraemon 2 - Hikari no Shinden (J) [!]"},
{0xA8275140, 0xB9B056E8,
"Doraemon 3 - Nobi Dai no Machi SOS! (J) [!]"},
{0x769D4D13, 0xDA233FFE,
"Dr. Mario 64 (U) [!]"},
{0xB6524461, 0xED6D04B1,
"Dual Heroes (E) [!]"},
{0x056EAB63, 0xC215FCD5,
"Dual Heroes (J) [!]"},
{0xA62230C3, 0xF0834488,
"Dual Heroes (U) [!]"},
{0xFBB9F1FA, 0x6BF88689,
"Duck Dodgers Starring Daffy Duck (U) [!]"},
{0xDC36626A, 0x3F3770CB,
"Duke Nukem - ZER0 H0UR (E) [!]"},
{0x32CA974B, 0xB2C29C50,
"Duke Nukem - ZER0 H0UR (F) [!]"},
{0x04DAF07F, 0x0D18E688,
"Duke Nukem - ZER0 H0UR (U) [!]"},
{0x57BFF74D, 0xDE747743,
"Duke Nukem 64 (E) [!]"},
{0x1E12883D, 0xD3B92718,
"Duke Nukem 64 (F)"},
{0xA273AB56, 0xDA33DB9A,
"Duke Nukem 64 (U) [!]"},
{0x492B9DE8, 0xC6CCC81C,
"Earthworm Jim 3D (E) [!]"},
{0xDF574191, 0x9EB5123D,
"Earthworm Jim 3D (U) [!]"},
{0x8C38E5DB, 0xB37C27D7,
"ECW Hardcore Revolution (E) [!]"},
{0xBDF9766D, 0xBD068D70,
"ECW Hardcore Revolution (U) [!]"},
{0xF2A653CB, 0x60633B3B,
"Elmo's Letter Adventure (U) [!]"},
{0x02B1538F, 0xC94B88D0,
"Elmo's Number Journey (U) [!]"},
{0xE13AE2DC, 0x4FB65CE8,
"Eltale Monsters (J) [!]"},
{0xAF754F7B, 0x1DD17381,
"Excitebike 64 - Kiosk (U) [!]"},
{0x202A8EE4, 0x83F88B89,
"Excitebike 64 (E) [!]"},
{0x861C3519, 0xF6091CE5,
"Excitebike 64 (J) [!]"},
{0x07861842, 0xA12EBC9F,
"Excitebike 64 (U) [!]"},
{0x8E9D834E, 0x1E8B29A9,
"Extreme-G (E) [!]"},
{0xEE802DC4, 0x690BD57D,
"Extreme-G (J) [!]"},
{0xFDA245D2, 0xA74A3D47,
"Extreme-G (U) [!]"},
{0x1185EC85, 0x4B5A7731,
"Extreme-G XG2 (E) [!]"},
{0x399B9B81, 0xD533AD11,
"Extreme-G XG2 (J) [!]"},
{0x5CD4150B, 0x470CC2F1,
"Extreme-G XG2 (U) [!]"},
{0xFDD248B2, 0x569A020E,
"F-1 Pole Position 64 (E) [!]"},
{0xAE82687A, 0x9A3F388D,
"F-1 Pole Position 64 (U) [!]"},
{0xCC3CC8B3, 0x0EC405A4,
"F-1 World Grand Prix (E) [!]"},
{0xB70BAEE5, 0x3A5005A8,
"F-1 World Grand Prix (F) [!]"},
{0x38442634, 0x66B3F060,
"F-1 World Grand Prix (G)"},
{0x64BF47C4, 0xF4BD22BA,
"F-1 World Grand Prix (J) [!]"},
{0xCC3CC8B3, 0x0EC405A4,
"F-1 World Grand Prix (U) [!]"},
{0x07C1866E, 0x5775CCDE,
"F-1 World Grand Prix II (E) [!]"},
{0x6DFF4C37, 0xB1B763FD,
"Famista 64 (J) [!]"},
{0x3CECBCB8, 0x6126BF07,
"F1 Racing Championship (E) [!]"},
{0x0E31EDF0, 0xC37249D5,
"FIFA - Road to World Cup 98 (E) [!]"},
{0xF5733C67, 0x17A3973A,
"FIFA - Road to World Cup 98 (J) [!]"},
{0xCB1ACDDE, 0xCF291DF2,
"FIFA - Road to World Cup 98 (U) [!]"},
{0x0198A651, 0xFC219D84,
"FIFA 99 (E) [!]"},
{0x7613A630, 0x3ED696F3,
"FIFA 99 (U) [!]"},
{0xC3F19159, 0x65D2BC5A,
"FIFA Soccer 64 (E) [!]"},
{0xC3F19159, 0x65D2BC5A,
"FIFA Soccer 64 (U) [!]"},
{0x36F1C74B, 0xF2029939,
"Fighter's Destiny (E) [!]"},
{0x0C41F9C2, 0x01717A0D,
"Fighter's Destiny (F) [!]"},
{0xFE94E570, 0xE4873A9C,
"Fighter's Destiny (G) [!]"},
{0x52F78805, 0x8B8FCAB7,
"Fighter's Destiny (U) [!]"},
{0xAEEF2F45, 0xF97E30F1,
"Fighter's Destiny 2 (U) [!]"},
{0x49E46C2D, 0x7B1A110C,
"Fighting Cup (J) [!]"},
{0x66CF0FFE, 0xAD697F9C,
"Fighting Force 64 (E) [!]"},
{0x32EFC7CB, 0xC3EA3F20,
"Fighting Force 64 (U) [!]"},
{0x28D5562D, 0xE4D5AE50,
"Fire Electric Pen (J) [!]"},
{0x22E9623F, 0xB60E52AD,
"Flying Dragon (E) [!]"},
{0xA92D52E5, 0x1D26B655,
"Flying Dragon (U) [!]"},
{0x142A17AA, 0x13028D96,
"Forsaken 64 (E) [!]"},
{0xC3CD76FF, 0x9B9DCBDE,
"Forsaken 64 (G) [!]"},
{0x9E330C01, 0x8C0314BA,
"Forsaken 64 (U) [!]"},
{0x3261D479, 0xED0DBC25,
"Fox Sports College Hoops '99 (U) [!]"},
{0xE8E5B179, 0x44AA30E8,
"Frogger 2 (U) (Unreleased Alpha)"},
{0xF774EAEE, 0xF0D8B13E,
"Fushigi no Dungeon - Furai no Shiren 2 (J) [!]"},
{0x776646F6, 0x06B9AC2B,
"F-Zero X (E) [!]"},
{0x4D3E622E, 0x9B828B4E,
"F-Zero X (J) [!]"},
{0xB30ED978, 0x3003C9F9,
"F-Zero X (U) [!]"},
{0x68FCF726, 0x49658CBC,
"G.A.S.P! Fighter's NEXTream (E) [!]"},
{0xAF8679B6, 0x5E1011BF,
"G.A.S.P! Fighter's NEXTream (J) [!]"},
{0x8F89ABA0, 0xC34C2610,
"GameShark V3.3 (Unl)"},
{0x832C168B, 0x56A2CDAE,
"Ganbare Goemon (J)"},
{0x457B9CD9, 0x09C55352,
"Ganbare Goemon 2 - Deorudero Douchuu Obake Tenkomori (J) [!]"},
{0xB2C6D27F, 0x2DA48CFD,
"Ganbare Goemon - Mononoke Sugoroku (J)"},
{0x21FFFD0A, 0xFA0D1D98,
"Ganbare Nippon Olympics 2000 (J) [!]"},
{0xD543BCD6, 0x2BA5E256,
"Gauntlet Legends (E) [!]"},
{0x70B0260E, 0x6716D04C,
"Gauntlet Legends (J) [!]"},
{0x729B5E32, 0xB728D980,
"Gauntlet Legends (U) [!]"},
{0x489C84E6, 0x4C6E49F9,
"Getter Love!! Cho Renai Party Game (J) [!]"},
{0x99179359, 0x2FE7EBC3,
"Gex 3 - Deep Cover Gecko (E) [!]"},
{0x874733A4, 0xA823745A,
"Gex 3 - Deep Cover Gecko (FG) [!]"},
{0x3EDC7E12, 0xE26C1CC9,
"Gex 3 - Deep Cover Gecko (U) [!]"},
{0xE68A000E, 0x639166DD,
"Gex 64 - Enter the Gecko (E) [!]"},
{0x89FED774, 0xCAAFE21B,
"Gex 64 - Enter the Gecko (U) [!]"},
{0xF5237301, 0x99E3EE93,
"Glover (E) [!]"},
{0x8E6E01FF, 0xCCB4F948,
"Glover (U) [!]"},
{0x4252A5AD, 0xAE6FBF4E,
"Goemon's Great Adventure (U) [!]"},
{0x4690FB1C, 0x4CD56D44,
"Golden Nugget 64 (U) [!]"},
{0x0414CA61, 0x2E57B8AA,
"GoldenEye 007 (E) [!]"},
{0xA24F4CF1, 0xA82327BA,
"GoldenEye 007 (J) [!]"},
{0xDCBC50D1, 0x09FD1AA3,
"GoldenEye 007 (U) [!]"},
{0xEE4A0E33, 0x8FD588C9,
"GT 64 Championship Edition (E) [!]"},
{0xC49ADCA2, 0xF1501B62,
"GT 64 Championship Edition (U) [!]"},
{0x95A80114, 0xE0B72A7F,
"Hamster Monogatari 64 (J)"},
{0x36F22FBF, 0x318912F2,
"Hanafuda 64 - Tenshi no Yakusoku (J) [!]"},
{0x09AE57B1, 0x182A5637,
"Harukanaru Augusta Masters 98 (J) [!]"},
{0x98DF9DFC, 0x6606C189,
"Harvest Moon 64 (U) [!]"},
{0x3E70E866, 0x4438BAE8,
"Heiwa Pachinko World (J) [!]"},
{0xAE90DBEB, 0x79B89123,
"Hercules - The Legendary Journeys (E) [!]"},
{0x7F3CEB77, 0x8981030A,
"Hercules - The Legendary Journeys (U) [!]"},
{0x95B2B30B, 0x2B6415C1,
"Hexen (E) [!]"},
{0x5C1B5FBD, 0x7E961634,
"Hexen (F)"},
{0x9AB3B50A, 0xBC666105,
"Hexen (G) [!]"},
{0x66751A57, 0x54A29D6E,
"Hexen (J) [!]"},
{0x9CAB6AEA, 0x87C61C00,
"Hexen (U) [!]"},
{0xD3F10E5D, 0x052EA579,
"Hey You, Pikachu! (U) [!]"},
{0x35FF8F1A, 0x6E79E3BE,
"Hiro no Ken Twin (J) [!]"},
{0x277B129D, 0xDD3879FF,
"Holy Magic Century (E) [!]"},
{0xB35FEBB0, 0x7427B204,
"Holy Magic Century (F)"},
{0x75FA0E14, 0xC9B3D105,
"Holy Magic Century (G) [!]"},
{0xC1D702BD, 0x6D416547,
"Hoshi no Kirby 64 (J) (v1.0)"},
{0xCA1BB86F, 0x41CCA5C5,
"Hoshi no Kirby 64 (J) (V1.1)"},
{0x0C581C7A, 0x3D6E20E4,
"Hoshi no Kirby 64 (J) (v1.2)"},
{0xBCB1F89F, 0x060752A2,
"Hoshi no Kirby 64 (J) (v1.3)"},
{0xE7D20193, 0xC1158E93,
"Hot Wheels Turbo Racing (E) [!]"},
{0xC7C98F8E, 0x42145DDE,
"Hot Wheels Turbo Racing (U) [!]"},
{0x72611D7D, 0x9919BDD2,
"HSV Adventure Racing (A)"},
{0x5535972E, 0xBD8E3295,
"Human Grand Prix (J) [!]"},
{0x641D3A7F, 0x86820466,
"Hybrid Heaven (E) [!]"},
{0x0DE2CE36, 0xD41D29E6,
"Hybrid Heaven (J) [!]"},
{0x102888BF, 0x434888CA,
"Hybrid Heaven (U) [!]"},
{0xB58988E9, 0xB1FC4BE8,
"Hydro Thunder (E) [!]"},
{0x29A045CE, 0xABA9060E,
"Hydro Thunder (F)"},
{0xC8DC65EB, 0x3D8C8904,
"Hydro Thunder (U) [!]"},
{0x69458B9E, 0xFC95F936,
"Iggy-kun no Bura Bura Poyon (J) [!]"},
{0xD692CC5E, 0xEC58D072,
"Iggy's Reckin' Balls (E) [!]"},
{0xE616B5BC, 0xC9658B88,
"Iggy's Reckin' Balls (U) [!]"},
{0xAF9DCC15, 0x1A723D88,
"Indiana Jones and the Infernal Machine (U) [!]"},
{0xE436467A, 0x82DE8F9B,
"Indy Racing 2000 (U) [!]"},
{0x8C138BE0, 0x95700E46,
"In-Fisherman Bass Hunter 64 (U) [!]"},
{0x336364A0, 0x06C8D5BF,
"International Superstar Soccer 2000 (E) [!]"},
{0xBAE8E871, 0x35FF944E,
"International Superstar Soccer 2000 (FI)"},
{0x8E835437, 0xCD5748B4,
"International Superstar Soccer 2000 (U) [!]"},
{0xE2D37CF0, 0xF57E4EAE,
"International Superstar Soccer 64 (E) [!]"},
{0x5F2763C4, 0x62412AE5,
"International Superstar Soccer 64 (U) [!]"},
{0xF41B6343, 0xC10661E6,
"International Superstar Soccer '98 (E) [!]"},
{0x7F0FDA09, 0x6061CE0B,
"International Superstar Soccer '98 (U) [!]"},
{0x20073BC7, 0x5E3B0111,
"International Track and Field 2000 (U) [!]"},
{0x6712C779, 0x3B72781D,
"International Track and Field Summer Games (E) [!]"},
{0x87766747, 0x91C27165,
"J. League Dynamite Soccer (J) [!]"},
{0x4FBFA429, 0x6920BB15,
"J. League Eleven Beat 1997 (J) [!]"},
{0x54554A42, 0xE4985FFB,
"J. League Live 64 (J)"},
{0xE8D29DA0, 0x15E61D94,
"J. League Tactics Soccer (J) [!]"},
{0xC73AD016, 0x48C5537D,
"Jangou Simulation Mahjong Do 64 (J) [!]"},
{0x69256460, 0xB9A3F586,
"Jeopardy! (U) [!]"},
{0x21F7ABFB, 0x6A8AA7E8,
"Jeremy McGrath Supercross 2000 (E) [!]"},
{0xBB30B1A5, 0xFCF712CE,
"Jeremy McGrath Supercross 2000 (U) [!]"},
{0xDFD8AB47, 0x3CDBEB89,
"Jet Force Gemini - Kiosk (U) [!]"},
{0x68D7A1DE, 0x0079834A,
"Jet Force Gemini (E) [!]"},
{0x8A6009B6, 0x94ACE150,
"Jet Force Gemini (U) [!]"},
{0xCDB8B4D0, 0x8832352D,
"Jet Force Gemini (U) [f1]"},
{0xAF19D9F5, 0xB70223CC,
"Jikkyou G1 Stable (J) [!]"},
{0x63112A53, 0xA29FA88F,
"Jikkyou J. League 1999 - Perfect Striker 2 (J) [!]"},
{0x146C4366, 0x72A6DEB3,
"Jikkyou J. League Perfect Striker (J) [!]"},
{0x0AC244D1, 0x1F0EC605,
"Jikkyou Powerful Pro Yakyuu 2000 (J) (V1.0) [!]"},
{0x4264DF23, 0xBE28BDF7,
"Jikkyou Powerful Pro Yakyuu 2000 (J) (V1.1) [!]"},
{0x6EDD4766, 0xA93E9BA8,
"Jikkyou Powerful Pro Baseball Basic 2001 (J) [!]"},
{0x3FEA5620, 0x7456DB40,
"Jikkyou World Cup France '98 (J) (v1.0)"},
{0xC954B38C, 0x6F62BEB3,
"Jikkyou World Cup France '98 (J) (v1.1)"},
{0xE1C7ABD6, 0x4E707F28,
"Jikkyou World Cup France '98 (J) (v1.2)"},
{0xE0A79F8C, 0x32CC97FA,
"Jikkyou World Soccer 3 (J) [!]"},
{0x4AAAF6ED, 0x376428AD,
"Jinsei Game 64 (J) [!]"},
{0x0F743195, 0xD8A6DB95,
"John Romero's Daikatana (E) [!]"},
{0x9D7E3C4B, 0xE60F4A6C,
"John Romero's Daikatana (J) [!]"},
{0xD0151AB0, 0xFE5CA14B,
"John Romero's Daikatana (U) [!]"},
{0x4F29474F, 0x30CB707A,
"Kakutou Denshou - F-Cup Maniax (J) [!]"},
{0x36281F23, 0x009756CF,
"Ken Griffey Jr.'s Slugfest (U) [!]"},
{0x979B263E, 0xF8470004,
"Killer Instinct Gold (E) [!]"},
{0x9E8FE2BA, 0x8B270770,
"Killer Instinct Gold (U) (V1.0) [!]"},
{0x9E8FCDFA, 0x49F5652B,
"Killer Instinct Gold (U) (V1.1) [!]"},
{0xF908CA4C, 0x36464327,
"Killer Instinct Gold (U) (V1.2) [!]"},
{0x519EA4E1, 0xEB7584E8,
"King Hill 64 - Extreme Snowboarding (J) [!]"},
{0x75BC6AD6, 0x78552BC9,
"Kiratto Kaiketsu! 64 Tanteidan (J) [!]"},
{0x0D93BA11, 0x683868A6,
"Kirby 64 - The Crystal Shards (E) [!]"},
{0x46039FB4, 0x0337822C,
"Kirby 64 - The Crystal Shards (U) [!]"},
{0x4A997C74, 0xE2087F99,
"Knife Edge - Nose Gunner (E) [!]"},
{0x931AEF3F, 0xEF196B90,
"Knife Edge - Nose Gunner (J) [!]"},
{0xFCE0D799, 0x65316C54,
"Knife Edge - Nose Gunner (U) [!]"},
{0xE3D6A795, 0x2A1C5D3C,
"Knockout Kings 2000 (E) [!]"},
{0x0894909C, 0xDAD4D82D,
"Knockout Kings 2000 (U) [!]"},
{0x1739EFBA, 0xD0B43A68,
"Kobe Bryant's NBA Courtside (E) [!]"},
{0x616B8494, 0x8A509210,
"Kobe Bryant's NBA Courtside (U) [!]"},
{0x5E3E60E8, 0x4AB5D495,
"Kuiki Uhabi Suigo (J) [!]"},
{0x7F304099, 0x52CF5276,
"Last Legion UX (J) [!]"},
{0xE97955C6, 0xBC338D38,
"Legend of Zelda 2, The - Majora's Mask (E) [!]"},
{0x0A5D8F83, 0x98C5371A,
"Legend of Zelda, The - Majora's Mask (E) (M4) (v1.1)"},
{0x5354631C, 0x03A2DEF0,
"Legend of Zelda 2, The - Majora's Mask (U) [!]"},
{0xB044B569, 0x373C1985,
"Legend of Zelda, The - Ocarina of Time (E) (V1.0) [!]"},
{0xB2055FBD, 0x0BAB4E0C,
"Legend of Zelda, The - Ocarina of Time (E) (M3) (V1.1)"},
{0x09465AC3, 0xF8CB501B,
"Legend of Zelda, The - Ocarina of Time (E) (GC Version)"},
{0xEC7011B7, 0x7616D72B,
"Legend of Zelda, The - Ocarina of Time (U) (V1.0) [!]"},
{0xD43DA81F, 0x021E1E19,
"Legend of Zelda, The - Ocarina of Time (U) (V1.1) [!]"},
{0x693BA2AE, 0xB7F14E9F,
"Legend of Zelda, The - Ocarina of Time (U) (V1.2) [!]"},
{0xF034001A, 0xAE47ED06,
"Legend of Zelda, The - Ocarina of Time - Master Quest (U) (GC Version)"},
{0xF3DD35BA, 0x4152E075,
"Legend of Zelda, The - Ocarina of Time (U) (GC Version)"},
{0x1D4136F3, 0xAF63EEA9,
"Legend of Zelda, The - Ocarina of Time - Master Quest (E) [h1C]"},
{0x1D4136F3, 0xAF63EEA9,
"Legend of Zelda, The - Ocarina of Time - Master Quest (E)"},
{0x27A3831D, 0xB505A533,
"Legend of Zelda, The - Ocarina of Time - Master Quest (E) [f1] (NTSC)"},
{0xB443EB08, 0x4DB31193,
"Legend of Zelda, The - Majora's Mask - Collector's Edition (U) (GC Version)"},
{0xB443EB08, 0x4DB31193,
"Legend of Zelda, The - Majora's Mask (U) (GC Version)"},
{0x917D18F6, 0x69BC5453,
"Legend of Zelda, The - Ocarina of Time - Master Quest (U) (Debug Rom)"},
{0x6AECEC4F, 0xF0924814,
"Legend of Zelda, The - Majora's Mask - Collector's Edition (E) (GC Version)"},
{0xBF799345, 0x39FF7A02,
"Legend of Zelda, The - Majora's Mask (U) (Demo)"},
{0xF478D8B3, 0x9716DD6D,
"LEGO Racers (E) [!]"},
{0x096A40EA, 0x8ABE0A10,
"LEGO Racers (U) [!]"},
{0x2B696CB4, 0x7B93DCD8,
"Les Razmoket - La Chasse Aux Tresors (F) [!]"},
{0x3D67C62B, 0x31D03150,
"Let's Smash Tennis (J) [!]"},
{0x60460680, 0x305F0E72,
"Lode Runner 3-D (E) [!]"},
{0x964ADD0B, 0xB29213DB,
"Lode Runner 3-D (J) [!]"},
{0x255018DF, 0x57D6AE3A,
"Lode Runner 3-D (U) [!]"},
{0x0AA0055B, 0x7637DF65,
"Looney Tunes - Duck Dodgers (E) [!]"},
{0x2483F22B, 0x136E025E,
"Lylat Wars (A) [!]"},
{0xF4CBE92C, 0xB392ED12,
"Lylat Wars (E) [!]"},
{0x1145443D, 0x11610EDB,
"Mace - The Dark Age (E) [!]"},
{0x6B700750, 0x29D621FE,
"Mace - The Dark Age (U) [!]"},
{0x0CB81686, 0x5FD85A81,
"Madden 2000 (U) [!]"},
{0xA197CB52, 0x7520DE0E,
"Madden Football 64 (E) [!]"},
{0x13836389, 0x265B3C76,
"Madden Football 64 (U) [!]"},
{0xEB38F792, 0x190EA246,
"Madden NFL 2001 (U) [!]"},
{0xD7134F8D, 0xC11A00B5,
"Madden NFL 2002 (U) [!]"},
{0x3925D625, 0x8C83C75E,
"Madden NFL 99 (E) [!]"},
{0xDEB78BBA, 0x52F6BD9D,
"Madden NFL 99 (U) [!]"},
{0xE4906679, 0x9F243F05,
"Magical Tetris Challenge (E) [!]"},
{0xE1EF93F7, 0x14908B0B,
"Magical Tetris Challenge (G) [!]"},
{0x80C8564A, 0x929C65AB,
"Magical Tetris Challenge (J) [!]"},
{0x75B61647, 0x7ADABF78,
"Magical Tetris Challenge (U) [!]"},
{0xC53EDC41, 0xECE19359,
"Mahjong 64 (KOEI) (J) [!]"},
{0xCCCC821E, 0x96E88A83,
"Mahjong Hourouki Classic (J) [!]"},
{0x0FC42C70, 0x8754F1CD,
"Mahjong Master (J) [!]"},
{0xCDB998BE, 0x1024A5C8,
"Major League Baseball Featuring Ken Griffey Jr. (E) [!]"},
{0x80C1C05C, 0xEA065EF4,
"Major League Baseball Featuring Ken Griffey Jr. (U) [!]"},
{0x9CCE5B1D, 0x6351E283,
"MAME 64 Emulator V1.0 (PD)"},
{0x62E957D0, 0x7FC15A5D,
"Mario Golf (E) [!]"},
{0x664BA3D4, 0x678A80B7,
"Mario Golf (U) [!]"},
{0xD48944D1, 0xB0D93A0E,
"Mario Golf 64 (J) [!]"},
{0xC3B6DE9D, 0x65D2DE76,
"Mario Kart 64 (E) (V1.0) [!]"},
{0x2577C7D4, 0xD18FAAAE,
"Mario Kart 64 (E) (V1.1) [!]"},
{0x6BFF4758, 0xE5FF5D5E,
"Mario Kart 64 (J) (V1.0) [!]"},
{0xC9C3A987, 0x5810344C,
"Mario Kart 64 (J) (V1.1) [!]"},
{0x3E5055B6, 0x2E92DA52,
"Mario Kart 64 (U) [!]"},
{0x9A9890AC, 0xF0C313DF,
"Mario no Photopie (J) [!]"},
{0x9C663069, 0x80F24A80,
"Mario Party (E) [!]"},
{0xADA815BE, 0x6028622F,
"Mario Party (J) [!]"},
{0x2829657E, 0xA0621877,
"Mario Party (U) [!]"},
{0x82380387, 0xDFC744D9,
"Mario Party 2 (E) [!]"},
{0xED567D0F, 0x38B08915,
"Mario Party 2 (J) [!]"},
{0x9EA95858, 0xAF72B618,
"Mario Party 2 (U) [!]"},
{0xC5674160, 0x0F5F453C,
"Mario Party 3 (E) [!]"},
{0x0B0AB4CD, 0x7B158937,
"Mario Party 3 (J) [!]"},
{0x7C3829D9, 0x6E8247CE,
"Mario Party 3 (U) [!]"},
{0x3BA7CDDC, 0x464E52A0,
"Mario Story (J) [!]"},
{0x839F3AD5, 0x406D15FA,
"Mario Tennis (E) [!]"},
{0x3A6C42B5, 0x1ACADA1B,
"Mario Tennis (J) [!]"},
{0x5001CF4F, 0xF30CB3BD,
"Mario Tennis (U) [!]"},
{0x0EC158F5, 0xFB3E6896,
"Mega Man 64 (U) [!]"},
{0x1001F10C, 0x3D51D8C1,
"Mia Hamm Soccer 64 (U) [!]"},
{0xE36166C2, 0x8613A2E5,
"Michael Owens WLS 2000 (E) [!]"},
{0x736AE6AF, 0x4117E9C7,
"Mickey no Racing Challenge USA (J) [!]"},
{0xDED0DD9A, 0xE78225A7,
"Mickey's Speedway USA (E) [!]"},
{0xFA8C4571, 0xBBE7F9C0,
"Mickey's Speedway USA (U) [!]"},
{0x2A49018D, 0xD0034A02,
"Micro Machines 64 Turbo (E) [!]"},
{0xF1850C35, 0xACE07912,
"Micro Machines 64 Turbo (U) [!]"},
{0xE4B35E4C, 0x1AC45CC9,
"Midway's Greatest Arcade Hits Volume 1 (U) [!]"},
{0x09D53E16, 0x3AB268B9,
"Mike Piazza's Strike Zone (U) [!]"},
{0x9A490D9D, 0x8F013ADC,
"Milo's Astro Lanes (E) [!]"},
{0x2E955ECD, 0xF3000884,
"Milo's Astro Lanes (U) [!]"},
{0x418BDA98, 0x248A0F58,
"Mischief Makers (E) [!]"},
{0x0B93051B, 0x603D81F9,
"Mischief Makers (U) [!]"},
{0x2256ECDA, 0x71AB1B9C,
"Mission Impossible (E) [!]"},
{0x20095B34, 0x343D9E87,
"Mission Impossible (F) [!]"},
{0x93EB3F7E, 0x81675E44,
"Mission Impossible (G) [!]"},
{0xEBA949DC, 0x39BAECBD,
"Mission Impossible (I) [!]"},
{0x5F6A04E2, 0xD4FA070D,
"Mission Impossible (S) [!]"},
{0x26035CF8, 0x802B9135,
"Mission Impossible (U) [!]"},
{0x28768D6D, 0xB379976C,
"Monaco Grand Prix (U) [!]"},
{0x5AC383E1, 0xD712E387,
"Monopoly (U) [!]"},
{0xD3D806FC, 0xB43AA2A8,
"Monster Truck Madness 64 (E) [!]"},
{0xB19AD999, 0x7E585118,
"Monster Truck Madness 64 (U) [!]"},
{0xE8E8DD70, 0x415DD198,
"Morita Shogi 64 (J) [!]"},
{0x73036F3B, 0xCE0D69E9,
"Mortal Kombat 4 (E) [!]"},
{0x417DD4F4, 0x1B482FE2,
"Mortal Kombat 4 (U) [!]"},
{0xFF44EDC4, 0x1AAE9213,
"Mortal Kombat Mythologies - Sub-Zero (E) [!]"},
{0xC34304AC, 0x2D79C021,
"Mortal Kombat Mythologies - Sub-Zero (U) [!]"},
{0x8C3D1192, 0xBEF172E1,
"Mortal Kombat Trilogy (E) [!]"},
{0xD9F75C12, 0xA8859B59,
"Mortal Kombat Trilogy (U) (V1.0) [!]"},
{0x83F33AA9, 0xA901D40D,
"Mortal Kombat Trilogy (U) (V1.2) [!]"},
{0xB8F0BD03, 0x4479189E,
"MRC - Multi Racing Championship (E) [!]"},
{0xA6B6B413, 0x15D113CC,
"MRC - Multi Racing Championship (J) [!]"},
{0x2AF9B65C, 0x85E2A2D7,
"MRC - Multi Racing Championship (U) [!]"},
{0x1938525C, 0x586E9656,
"Ms. Pac-Man Maze Madness (U) [!]"},
{0xF5360FBE, 0x2BF1691D,
"Mystical Ninja - Starring Goemon (E) [!]"},
{0xFCBCCB21, 0x72903C6B,
"Mystical Ninja - Starring Goemon (U) [!]"},
{0x7F9345D3, 0x841ECADE,
"Mystical Ninja 2 - Starring Goemon (E) [!]"},
{0xAE2D3A35, 0x24F0D41A,
"Nagano Olympic Hockey '98 (E) [!]"},
{0x7EC22587, 0xEF1AE323,
"Nagano Olympic Hockey '98 (U) [!]"},
{0x90F43037, 0x5C5370F5,
"Nagano Olympic Hockey '98 (J) [!]"},
{0x6D452016, 0x713C09EE,
"Nagano Winter Olympics '98 (E) [!]"},
{0x2FC5C34C, 0x7A05CC9D,
"Nagano Winter Olympics '98 (J) [!]"},
{0x8D2BAE98, 0xD73725BF,
"Nagano Winter Olympics '98 (U) [!]"},
{0x5129B6DA, 0x9DEF3C8C,
"Namco Museum 64 (U) [!]"},
{0xDF331A18, 0x5FD4E044,
"NASCAR 2000 (U) [!]"},
{0xAE4992C9, 0x9253B253,
"NASCAR 99 (E) [!]"},
{0x23749578, 0x80DC58FD,
"NASCAR 99 (U) [!]"},
{0x916852D8, 0x73DBEAEF,
"NBA Courtside 2 - Featuring Kobe Bryant (U) [!]"},
{0xC788DCAE, 0xBD03000A,
"NBA Hangtime (E) [!]"},
{0x4E69B487, 0xFE18E290,
"NBA Hangtime (U) [!]"},
{0xAAE11F01, 0x2625A045,
"NBA In the Zone 2 (J) [!]"},
{0xB3054F9F, 0x96B69EB5,
"NBA In the Zone 2000 (E) [!]"},
{0x8DF95B18, 0xECDA497B,
"NBA In the Zone 2000 (U) [!]"},
{0x36ACBA9B, 0xF28D4D94,
"NBA In the Zone '98 (J) [!]"},
{0x6A121930, 0x665CC274,
"NBA In the Zone '98 (U) [!]"},
{0xA292524F, 0x3D6C2A49,
"NBA In the Zone '99 (U) [!]"},
{0xB6D0CAA0, 0xE3F493C8,
"NBA Jam 2000 (E) [!]"},
{0xEBEEA8DB, 0xF2ECB23C,
"NBA Jam 2000 (U) [!]"},
{0xE600831E, 0x59F422A8,
"NBA Jam 99 (E) [!]"},
{0x810729F6, 0xE03FCFC1,
"NBA Jam 99 (U) [!]"},
{0xEB499C8F, 0xCD4567B6,
"NBA Live 2000 (E) [!]"},
{0x5F25B0EE, 0x6227C1DB,
"NBA Live 2000 (U) [!]"},
{0xCF84F45F, 0x00E4F6EB,
"NBA Live 99 (E) [!]"},
{0x57F81C9B, 0x1133FA35,
"NBA Live 99 (U) [!]"},
{0xACDE962F, 0xB2CBF87F,
"NBA Pro 98 (E) [!]"},
{0x8D1780B6, 0x57B3B976,
"NBA Pro 99 (E) [!]"},
{0x3FFE80F4, 0xA7C15F7E,
"NBA Showtime - NBA on NBC (U) [!]"},
{0x147E0EDB, 0x36C5B12C,
"Neon Genesis Evangelion (J) [!]"},
{0xE61CFF0A, 0xCE1C0D71,
"New Tetris, The (E) [!]"},
{0x2153143F, 0x992D6351,
"New Tetris, The (U) [!]"},
{0x30EAD54F, 0x31620BF6,
"NFL Blitz - Special Edition (U) [!]"},
{0xD094B170, 0xD7C4B5CC,
"NFL Blitz (U) [!]"},
{0x15A00969, 0x34E5A285,
"NFL Blitz 2000 (U) [!]"},
{0x36FA35EB, 0xE85E2E36,
"NFL Blitz 2001 (U) [!]"},
{0x88BD5A9E, 0xE81FDFBF,
"NFL Quarterback Club 2000 (E) [!]"},
{0xE3AB4ED0, 0x83040DD2,
"NFL Quarterback Club 2000 (U) [!]"},
{0x28784622, 0xFFB22985,
"NFL Quarterback Club 2001 (U) [!]"},
{0x4B629EF4, 0x99B21D9B,
"NFL Quarterback Club 98 (E) [!]"},
{0xD89BE2F8, 0x99C97ADF,
"NFL Quarterback Club 98 (U) [!]"},
{0x52A3CF47, 0x4EC13BFC,
"NFL Quarterback Club 99 (E) [!]"},
{0xBE76EDFF, 0x20452D09,
"NFL Quarterback Club 99 (U) [!]"},
{0x287D601E, 0xABF4F8AE,
"NHL 99 (E) [!]"},
{0x591A806E, 0xA5E6921D,
"NHL 99 (U) [!]"},
{0x82EFDC30, 0x806A2461,
"NHL Blades of Steel '99 (U) [!]"},
{0x29CE7692, 0x71C58579,
"NHL Breakaway 98 (E) [!]"},
{0x6DFDCDC3, 0x4DE701C8,
"NHL Breakaway 98 (U) [!]"},
{0x874621CB, 0x0031C127,
"NHL Breakaway 99 (E) [!]"},
{0x441768D0, 0x7D73F24F,
"NHL Breakaway 99 (U) [!]"},
{0xA9895CD9, 0x7020016C,
"NHL Pro 99 (E) [!]"},
{0x2857674D, 0xCC4337DA,
"Nightmare Creatures (U) [!]"},
{0xCD3C3CDF, 0x317793FA,
"Nintama Rantarou 64 (J)"},
{0x8A97A197, 0x272DF6C1,
"Nuclear Strike 64 (E) [!]"},
{0x8F50B845, 0xD729D22F,
"Nuclear Strike 64 (G) [!]"},
{0x4998DDBB, 0xF7B7AEBC,
"Nuclear Strike 64 (U) [!]"},
{0x5B9B1618, 0x1B43C649,
"Nushi Tsuri 64 - Shiokaze ni Notte (J) [!]"},
{0xD83BB920, 0xCC406416,
"Nushi Tsuri 64 (J) [!]"},
{0xE86415A6, 0x98395B53,
"O.D.T (Or Die Trying) (E) (M5) (Unreleased Final)"},
{0x2655BB70, 0x667D9925,
"O.D.T (Or Die Trying) (U) (M3) (Unreleased Final)"},
{0x812289D0, 0xC2E53296,
"Off Road Challenge (E) [!]"},
{0x319093EC, 0x0FC209EF,
"Off Road Challenge (U) [!]"},
{0x0375CF67, 0x56A93FAA,
"Ogre Battle 64 - Person of Lordly Caliber (J) (V1.1) [!]"},
{0xE6419BC5, 0x69011DE3,
"Ogre Battle 64 - Person of Lordly Caliber (U) [!]"},
{0xDC649466, 0x572FF0D9,
"Onegai Monsters (J) [!]"},
{0x74554B3B, 0xF4AEBCB5,
"Pachinko 365 Nichi (J) [!]"},
{0x19AB29AF, 0xC71BCD28,
"Paper Mario (E) [!]"},
{0x65EEE53A, 0xED7D733C,
"Paper Mario (U) [!]"},
{0xAC976B38, 0xC3A9C97A,
"Paperboy (E) [!]"},
{0x3E198D9E, 0xF2E1267E,
"Paperboy (U) [!]"},
{0xCFE2CB31, 0x4D6B1E1D,
"Parlor! Pro 64 Pachinko Jikki Simulation Game (J) [!]"},
{0x34495BAD, 0x502E9D26,
"Jikkyou Powerful Pro Yakyuu 4 (J) [!]"},
{0xD7891F1C, 0xC3E43788,
"Jikkyou Powerful Pro Yakyuu 4 (J) (V1.1) [!]"},
{0xD22943DA, 0xAC2B77C0,
"Jikkyou Powerful Pro Yakyuu 5 (J) [!]"},
{0xB75E20B7, 0xB3FEFDFD,
"Jikkyou Powerful Pro Yakyuu (J) [!]"},
{0xF468118C, 0xE32EE44E,
"PD Ultraman Battle Collection 64 (J) [!]"},
{0xC83CEB83, 0xFDC56219,
"Penny Racers (E) [!]"},
{0x73ABB1FB, 0x9CCA6093,
"Penny Racers (U) [!]"},
{0xE4B08007, 0xA602FF33,
"Perfect Dark (E) [!]"},
{0x96747EB4, 0x104BB243,
"Perfect Dark (J) [!]"},
{0xDDF460CC, 0x3CA634C0,
"Perfect Dark (U) (V1.0) [!]"},
{0x41F2B98F, 0xB458B466,
"Perfect Dark (U) (V1.1) [!]"},
{0xEE08C602, 0x6BC2D5A6,
"PGA European Tour (E) [!]"},
{0xB54CE881, 0xBCCB6126,
"PGA European Tour (U) [!]"},
{0x3F245305, 0xFC0B74AA,
"Pikachu Genki Dechu (J) [!]"},
{0x1AA05AD5, 0x46F52D80,
"Pilotwings 64 (E) [!]"},
{0x09CC4801, 0xE42EE491,
"Pilotwings 64 (J) [!]"},
{0xC851961C, 0x78FCAAFA,
"Pilotwings 64 (U) [!]"},
{0xEC0F690D, 0x32A7438C,
"Pocket Monsters Snap (J) [!]"},
{0x665E8259, 0xD098BD1D,
"Pocket Monsters Stadium (J) [!]"},
{0x63775886, 0x5FB80E7B,
"Pocket Monsters Stadium 2 (J) [!]"},
{0x4A1CD153, 0xD830AEF8,
"Pokemon Puzzle League (E) [!]"},
{0x3EB2E6F3, 0x062F9EFE,
"Pokemon Puzzle League (F) [!]"},
{0x7A4747AC, 0x44EEEC23,
"Pokemon Puzzle League (G) [!]"},
{0x19C553A7, 0xA70F4B52,
"Pokemon Puzzle League (U) [!]"},
{0x7BB18D40, 0x83138559,
"Pokemon Snap (A) [!]"},
{0x4FF5976F, 0xACF559D8,
"Pokemon Snap (E) [!]"},
{0xBA6C293A, 0x9FAFA338,
"Pokemon Snap (F) [!]"},
{0x5753720D, 0x2A8A884D,
"Pokemon Snap (G) [!]"},
{0xC0C85046, 0x61051B05,
"Pokemon Snap (I) [!]"},
{0x817D286A, 0xEF417416,
"Pokemon Snap (S) [!]"},
{0xCA12B547, 0x71FA4EE4,
"Pokemon Snap (U) [!]"},
{0x39119872, 0x07722E9F,
"Pokemon Snap Station (U) [!]"},
{0x1A122D43, 0xC17DAF0F,
"Pokemon Stadium - Kiosk (U) (V1.1) [!]"},
{0x84077275, 0x57315B9C,
"Pokemon Stadium (E) [!]"},
{0x91C9E05D, 0xAD3AAFB9,
"Pokemon Stadium (E) (V1.1)"},
{0xA23553A3, 0x42BF2D39,
"Pokemon Stadium (F) [!]"},
{0x42011E1B, 0xE3552DB5,
"Pokemon Stadium (G) [!]"},
{0xA53FA82D, 0xDAE2C15D,
"Pokemon Stadium (I)"},
{0xB6E549CE, 0xDC8134C0,
"Pokemon Stadium (S) [!]"},
{0x90F5D9B3, 0x9D0EDCF0,
"Pokemon Stadium (U) [!]"},
{0x2952369C, 0xB6E4C3A8,
"Pokemon Stadium 2 (E) [!]"},
{0xAC5AA5C7, 0xA9B0CDC3,
"Pokemon Stadium 2 (F) [!]"},
{0x439B7E7E, 0xC1A1495D,
"Pokemon Stadium 2 (G) [!]"},
{0xEFCEAF00, 0x22094848,
"Pokemon Stadium 2 (I)"},
{0xD0A1FC5B, 0x2FB8074B,
"Pokemon Stadium 2 (S) [!]"},
{0x03571182, 0x892FD06D,
"Pokemon Stadium 2 (U) [!]"},
{0xEE4FD7C2, 0x9CF1D938,
"Pokemon Stadium GS (J) [!]"},
{0x41380792, 0xA167E045,
"Polaris Sno Cross (U) [!]"},
{0xD7A6DCFA, 0xCCFEB6B7,
"Power League Baseball 64 (J) [!]"},
{0x39F60C11, 0xAB2EBA7D,
"Power Rangers - Lightspeed Rescue (E) [!]"},
{0xCF8957AD, 0x96D57EA9,
"Power Rangers - Lightspeed Rescue (U) [!]"},
{0xFC74D475, 0x9A0278AB,
"Powerpuff Girls, The - Chemical X-traction (U) [!]"},
{0xF3D27F54, 0xC111ACF9,
"Premier Manager 64 (E) [!]"},
{0x9BA10C4E, 0x0408ABD3,
"Pro Mahjong Kiwame 64 (J) [!]"},
{0x1BDCB30F, 0xA132D876,
"Pro Mahjong Tsuwamono 64 (J)"},
{0x4B4672B9, 0x2DBE5DE7,
"Puyo Puyo 4 - Puyo Puyo Party (J) [!]"},
{0x94807E6B, 0x60CC62E4,
"Puyo Puyo Sun 64 (J) [!]"},
{0xC0DE0747, 0xA2DF72D3,
"Puzzle Bobble 64 (J) [!]"},
{0x4BBBA2E2, 0x8BF3BBB2,
"Puzzle Master 64 by Michael Searl (PD)"},
{0x16931D74, 0x65DC6D34,
"Quake 64 (E) [!]"},
{0x9F5BF79C, 0xD2FE08A0,
"Quake 64 (U) [!]"},
{0x7433D9D7, 0x2C4322D0,
"Quake II (E) [!]"},
{0xBDA8F143, 0xB1AF2D62,
"Quake II (U) [!]"},
{0xC8BB4DD9, 0xCC5F430B,
"Quest 64 (U) [!]"},
{0x28705FA5, 0xB509690E,
"Racing Simulation - Monaco Grand Prix (E) [!]"},
{0x2877AC2D, 0xC3DC139A,
"Racing Simulation 2 (G) [!]"},
{0x67D21868, 0xC5424061,
"Rakuga Kids (E) [!]"},
{0x9F1ECAF0, 0xEEC48A0E,
"Rakuga Kids (J) [!]"},
{0x35D9BA0C, 0xDF485586,
"Rally '99 (J) [!]"},
{0x73A88E3D, 0x3AC5C571,
"Rally Challenge 2000 (U) [!]"},
{0x84D44448, 0x67CA19B0,
"Rampage - World Tour (E) [!]"},
{0xC29FF9E4, 0x264BFE7D,
"Rampage - World Tour (U) [!]"},
{0x5DFC4249, 0x99529C07,
"Rampage 2 - Universal Tour (E) [!]"},
{0x673D099B, 0xA4C808DE,
"Rampage 2 - Universal Tour (U) [!]"},
{0x20FD0BF1, 0xF5CF1D87,
"Rat Attack (E) [!]"},
{0x0304C48E, 0xAC4001B8,
"Rat Attack (U) [!]"},
{0x60D5E10B, 0x8BEDED46,
"Rayman 2 - The Great Escape (E) [!]"},
{0xF3C5BF9B, 0x160F33E2,
"Rayman 2 - The Great Escape (U) [!]"},
{0x3918834A, 0x15B50C29,
"Razor Freestyle Scooter (U) [!]"},
{0x8BD4A334, 0x1E138B05,
"Ready 2 Rumble Boxing (E) [!]"},
{0xEAB7B429, 0xBAC92C57,
"Ready 2 Rumble Boxing (U) [!]"},
{0xE9219533, 0x13FBAFBD,
"Ready 2 Rumble Boxing Round 2 (U) [!]"},
{0x9B500E8E, 0xE90550B3,
"Resident Evil 2 (E) [!]"},
{0xAA18B1A5, 0x07DB6AEB,
"Resident Evil 2 (U) [!]"},
{0xC3E7E29E, 0x5D7251CC,
"Re-Volt (E) [!]"},
{0x0F1FA987, 0xBFC1AFA6,
"Re-Volt (U) [!]"},
{0x02D8366A, 0x6CABEF9C,
"Road Rash 64 (E) [!]"},
{0xF050746C, 0x247B820B,
"Road Rash 64 (U) [!]"},
{0x74E87A70, 0x6293AED4,
"Roadsters (E) [!]"},
{0x0B6B4DDB, 0x9671E682,
"Roadsters (U) [!]"},
{0x272B690F, 0xAD0A7A77,
"Robot Ponkottsu 64 - Caramel of the 7 Seas (J) [!]"},
{0x75A0B893, 0x4CA321B5,
"Robotech - Crystal Dreams (U) (beta)"},
{0x9FF69D4F, 0x195F0059,
"Robotron 64 (E) [!]"},
{0xAC8E4B32, 0xE7B47326,
"Robotron 64 (U) [!]"},
{0x9FD375F8, 0x45F32DC8,
"Rocket - Robot on Wheels (E) [!]"},
{0x0C5EE085, 0xA167DD3E,
"Rocket - Robot on Wheels (U) [!]"},
{0xD666593B, 0xD7A25C07,
"Rockman Dash (J) [!]"},
{0xFEE97010, 0x4E94A9A0,
"RR64 - Ridge Racer 64 (E) [!]"},
{0x2500267E, 0x2A7EC3CE,
"RR64 - Ridge Racer 64 (U) [!]"},
{0x658F8F37, 0x1813D28D,
"RTL World League Soccer 2000 (G) [!]"},
{0x0C02B3C5, 0x9E2511B8,
"Rugrats - Scavenger Hunt (U) [!]"},
{0x4D3ADFDA, 0x7598FCAE,
"Rugrats - Treasure Hunt (E) [!]"},
{0x451ACA0F, 0x7863BC8A,
"Rugrats - Die große Schatzsuche (G)"},
{0x0AC61D39, 0xABFA03A6,
"Rugrats in Paris - The Movie (E) [!]"},
{0x1FC21532, 0x0B6466D4,
"Rugrats in Paris - The Movie (U) [!]"},
{0xB7CF2136, 0xFA0AA715,
"Rush 2 - Extreme Racing USA (E) [!]"},
{0xEDD6E031, 0x68136013,
"Rush 2 - Extreme Racing USA (U) [!]"},
{0x918E2D60, 0xF865683E,
"S.C.A.R.S. (E) [!]"},
{0x769147F3, 0x2033C10E,
"S.C.A.R.S. (U) [!]"},
{0x61D116B0, 0xFA24D60C,
"San Francisco Rush - Extreme Racing (E) [!]"},
{0x2A6B1820, 0x6ABCF466,
"San Francisco Rush - Extreme Racing (U) [!]"},
{0x51D29418, 0xD5B46AE3,
"San Francisco Rush 2049 (E) [!]"},
{0xB9A9ECA2, 0x17AAE48E,
"San Francisco Rush 2049 (U) [!]"},
{0xE3BD221D, 0x3C0834D3,
"Scooby-Doo - Classic Creep Capers (E) [!]"},
{0x0C814EC4, 0x58FE5CA8,
"Scooby-Doo - Classic Creep Capers (U) [!]"},
{0xEBF5F6B7, 0xC956D739,
"SD Hiryuu no Ken Densetsu (J) [!]"},
{0x60C437E5, 0xA2251EE3,
"Shadow Man (E) [!]"},
{0xEA06F8C3, 0x07C2DEED,
"Shadow Man (F) [!]"},
{0x84D5FD75, 0xBBFD3CDF,
"Shadow Man (G) [!]"},
{0x3A4760B5, 0x2D74D410,
"Shadow Man (U) [!]"},
{0xD84EEA84, 0x45B2F1B4,
"Shadowgate 64 - Trials of the Four Towers (E) [!]"},
{0x2BC1FCF2, 0x7B9A0DF4,
"Shadowgate 64 - Trials of the Four Towers (FGD) [!]"},
{0x02B46F55, 0x61778D0B,
"Shadowgate 64 - Trials of the Four Towers (IS)"},
{0xCCEDB696, 0xD3883DB4,
"Shadowgate 64 - Trials of the Four Towers (J) [!]"},
{0x036897CE, 0xE0D4FA54,
"Shadowgate 64 - Trials of the Four Towers (U) [!]"},
{0xB1D5280C, 0x4BA7BC2A,
"Sim City 2000 (J) [!]"},
{0xB6BC0FB0, 0xE3812198,
"Sin and Punishment - Tsumi To Batsu (J) [!]"},
{0x2EF4D519, 0xC64A0C5E,
"Snow Speeder (J) [!]"},
{0x5FD7CDA0, 0xD9BB51AD,
"Snowboard Kids (E) [!]"},
{0xDBF4EA9D, 0x333E82C0,
"Snowboard Kids (U) [!]"},
{0x84FC04FF, 0xB1253CE9,
"Snowbow Kids (J) [!]"},
{0xC2751D1A, 0xF8C19BFF,
"Snowboard Kids 2 (E) [!]"},
{0x930C29EA, 0x939245BF,
"Snowboard Kids 2 (U) [!]"},
{0x22212351, 0x4046594B,
"Sonic Wings Assault (J) [!]"},
{0xC00CA948, 0x8E60D34B,
"South Park - Chef's Luv Shack (E) [!]"},
{0xC00CA948, 0x8E60D34B,
"South Park - Chef's Luv Shack (U) [!]"},
{0x20B53662, 0x7B61899F,
"South Park (E) [!]"},
{0x91B66D42, 0x16AC4E46,
"South Park (G) [!]"},
{0x7ECBE939, 0x3C331795,
"South Park (U) [!]"},
{0x4F8AFC3A, 0xF7912DF2,
"South Park Rally (E) [!]"},
{0x07F3B276, 0xEC8F3D39,
"South Park Rally (U) [!]"},
{0x37463412, 0xEAC5388D,
"Space Dynamites (J) [!]"},
{0xEBFE2397, 0xFF74DA34,
"Space Invaders (U) [!]"},
{0xFC70E272, 0x08FFE7AA,
"Spacestation Silicon Valley (E)/(F) [!]"},
{0xBFE23884, 0xEF48EAAF,
"Spacestation Silicon Valley (J) [!]"},
{0xBFE23884, 0xEF48EAAF,
"Spacestation Silicon Valley (U) [!]"},
{0xA60ED171, 0x3D85D06E,
"Spider-Man (U) [!]"},
{0x0DED0568, 0x1502515E,
"St. Andrews Old Course (J) [!]"},
{0xFFCAA7C1, 0x68858537,
"Star Fox 64 (J) [!]"},
{0xA7D015F8, 0x2289AA43,
"Star Fox 64 (U) [!]"},
{0xBA780BA0, 0x0F21DB34,
"Star Fox 64 (U) (V1.1)"},
{0xB703EB23, 0x28AAE53A,
"Star Soldier Vanishing Earth (J) [!]"},
{0xDDD93C85, 0xDAE381E8,
"Star Soldier Vanishing Earth (U) [!]"},
{0xF163A242, 0xF2449B3B,
"Star Twins (J) [!]"},
{0x7EE0E8BB, 0x49E411AA,
"Star Wars - Rogue Squadron (E) [!]"},
{0x219191C1, 0x33183C61,
"Star Wars - Rogue Squadron (E) (V1.1)"},
{0x66A24BEC, 0x2EADD94F,
"Star Wars - Rogue Squadron (U) [!]"},
{0x4D486681, 0xAB7D9245,
"Star Wars - Shadows of the Empire (E) [!]"},
{0xFE24AC63, 0x1B41AA17,
"Star Wars - Shadows of the Empire (J) [!]"},
{0x264D7E5C, 0x18874622,
"Star Wars - Shadows of the Empire (U) (V1.0) [!]"},
{0x4147B091, 0x63251060,
"Star Wars - Shadows of the Empire (U) (V1.1) [!]"},
{0x4DD7ED54, 0x74F9287D,
"Star Wars - Shadows of the Empire (U) (V1.2) [!]"},
{0x827E4890, 0x958468DC,
"Star Wars - Shuggeki Rogue Chitai (J) [!]"},
{0xEAE6ACE2, 0x020B4384,
"Star Wars Episode I - Battle for Naboo (E) [!]"},
{0x3D02989B, 0xD4A381E2,
"Star Wars Episode I - Battle for Naboo (U) [!]"},
{0x53ED2DC4, 0x06258002,
"Star Wars Episode I - Racer (E) [!]"},
{0x61F5B152, 0x046122AB,
"Star Wars Episode I - Racer (J) [!]"},
{0x72F70398, 0x6556A98B,
"Star Wars Episode I - Racer (U) [!]"},
{0xBC9B2CC3, 0x4ED04DA5,
"StarCraft 64 (E) [!]"},
{0x42CF5EA3, 0x9A1334DF,
"StarCraft 64 (E) [!]"},
{0x0684FBFB, 0x5D3EA8A5,
"StarCraft 64 (U) [!]"},
{0xBC9B2CC3, 0x4ED04DA5,
"StarCraft 64 (Beta)"},
{0xD89E0E55, 0xB17AA99A,
"Starshot - Space Circus Fever (E) [!]"},
{0x94EDA5B8, 0x8673E903,
"Starshot - Space Circus Fever (U) [!]"},
{0x9510D8D7, 0x35100DD2,
"Stunt Racer 64 (U) [!]"},
{0xF4646B69, 0xC5751095,
"Super B-Daman - Battle Phoenix 64 (J) [!]"},
{0xF3F2F385, 0x6E490C7F,
"Super Bowling (J) [!]"},
{0xAA1D215A, 0x91CBBE9A,
"Super Bowling 64 (U) [!]"},
{0xA03CF036, 0xBCC1C5D2,
"Super Mario 64 (E) [!]"},
{0x4EAA3D0E, 0x74757C24,
"Super Mario 64 (J) [!]"},
{0x635A2BFF, 0x8B022326,
"Super Mario 64 (U) [!]"},
{0xD6FBA4A8, 0x6326AA2C,
"Super Mario 64 Shindou Edition (J) [!]"},
{0x66572080, 0x28E348E1,
"Super Robot Spirits (J) [!]"},
{0x1649D810, 0xF73AD6D2,
"Super Robot Taisen 64 (J) [!]"},
{0xDD26FDA1, 0xCB4A6BE3,
"Super Smash Bros. (A) [!]"},
{0x93945F48, 0x5C0F2E30,
"Super Smash Bros. (E) [!]"},
{0x916B8B5B, 0x780B85A4,
"Super Smash Bros. (U) [!]"},
{0x9CE02E22, 0x206EF1B0,
"Super Speed Race 64 (J) [!]"},
{0x2CBB127F, 0x09C2BFD8,
"Supercross 2000 (E) [!]"},
{0xC1452553, 0x5D7B24D9,
"Supercross 2000 (U) [!]"},
{0xB44CAB74, 0x07029A29,
"Superman (E) [!]"},
{0xA2E8F35B, 0xC9DC87D9,
"Superman (U) [!]"},
{0x35E811F3, 0x99792724,
"Susume! Taisen Puzzle Dama Toukon! Marumata Chou (J) [!]"},
{0xAEBCDD54, 0x15FF834A,
"Taz Express (E) [!]"},
{0xD9042FBB, 0xFCFF997C,
"Telefoot Soccer 2000 (F) [!]"},
{0x963ADBA6, 0xF7D5C89B,
"Tetris 64 (J) [!]"},
{0x0FE684A9, 0x8BB77AC4,
"Tetrisphere (E) [!]"},
{0x3C1FDABE, 0x02A4E0BA,
"Tetrisphere (U) [!]"},
{0xE0C4F72F, 0x769E1506,
"Tigger's Honey Hunt (E) [!]"},
{0x4EBFDD33, 0x664C9D84,
"Tigger's Honey Hunt (U) [!]"},
{0x2B4F4EFB, 0x43C511FE,
"Tom and Jerry in Fists of Furry (E) [!]"},
{0x63E7391C, 0xE6CCEA33,
"Tom and Jerry in Fists of Furry (U) [!]"},
{0x4875AF3D, 0x9A66D3A2,
"Tom Clancy's Rainbow Six (E) [!]"},
{0x486BF335, 0x034DCC81,
"Tom Clancy's Rainbow Six (F)"},
{0x8D412933, 0x588F64DB,
"Tom Clancy's Rainbow Six (G) [!]"},
{0x392A0C42, 0xB790E77D,
"Tom Clancy's Rainbow Six (U) [!]"},
{0x093F916E, 0x4408B698,
"Tonic Trouble (E) [!]"},
{0xEF9E9714, 0xC03B2C7D,
"Tonic Trouble (U) (V1.1) [!]"},
{0x9F8926A5, 0x0587B409,
"Tony Hawk's Pro Skater (E) [!]"},
{0x204EC022, 0xB119D185,
"Tony Hawk's Pro Skater (U) [!]"},
{0xE0144180, 0x650B78C9,
"Tony Hawk's Pro Skater (V1.1) [!]"},
{0x84EAB557, 0xC88A190F,
"Tony Hawk's Pro Skater 2 (E) [!]"},
{0x99150E18, 0x1266E6A5,
"Tony Hawk's Pro Skater 2 (U) [!]"},
{0x1A7F70B5, 0x00B7B9FD,
"Tony Hawk's Pro Skater 3 (U)"},
{0x5F3F49C6, 0x0DC714B0,
"Top Gear Hyper-Bike (E) [!]"},
{0x845B0269, 0x57DE9502,
"Top Gear Hyper-Bike (J) [!]"},
{0x8ECC02F0, 0x7F8BDE81,
"Top Gear Hyper-Bike (U) [!]"},
{0xD09BA538, 0x1C1A5489,
"Top Gear Overdrive (E) [!]"},
{0x0578F24F, 0x9175BF17,
"Top Gear Overdrive (J) [!]"},
{0xD741CD80, 0xACA9B912,
"Top Gear Overdrive (U) [!]"},
{0x7F43E701, 0x536328D1,
"Top Gear Rally (E) [!]"},
{0x0E596247, 0x753D4B8B,
"Top Gear Rally (J) [!]"},
{0x62269B3D, 0xFE11B1E8,
"Top Gear Rally (U) [!]"},
{0xBEBAB677, 0x51B0B5E4,
"Top Gear Rally 2 (E) [!]"},
{0xF82DD377, 0x8C3FB347,
"TG Rally 2 (UK Version)"},
{0xCFEF2CD6, 0xC9E973E6,
"Top Gear Rally 2 (J) [!]"},
{0xBE5973E0, 0x89B0EDB8,
"Top Gear Rally 2 (U) [!]"},
{0xEF703CA4, 0x4D4A9AC9,
"Toukon Road - Brave Spirits (J) [!]"},
{0x551C7F43, 0x9149831C,
"Toukon Road 2 - The Next Generation (J) [!]"},
{0xCCEB3858, 0x26952D97,
"Toy Story 2 (E) [!]"},
{0xCB93DB97, 0x7F5C63D5,
"Toy Story 2 (F) [!]"},
{0x782A9075, 0xE552631D,
"Toy Story 2 (G) [!]"},
{0xA150743E, 0xCF2522CD,
"Toy Story 2 (U) [!]"},
{0xFE4B6B43, 0x081D29A7,
"Triple Play 2000 (U) [!]"},
{0x2F7009DD, 0xFC3BAC53,
"Turok - Dinosaur Hunter (E) (V1.0) [!]"},
{0x665F09DD, 0xFC3BAC53,
"Turok - Dinosaur Hunter (E) (V1.0) [a1]"},
{0x2F700DCD, 0x176CC5C9,
"Turok - Dinosaur Hunter (E) (V1.1)/(V1.2) [!]"},
{0x665FD963, 0xB5CC6612,
"Turok - Dinosaur Hunter (G) [!]"},
{0x916AE6B8, 0x8817AB22,
"Turok - Dinosaur Hunter (J) [!]"},
{0x2F70F10D, 0x5C4187FF,
"Turok - Dinosaur Hunter (U) (V1.0) [!]"},
{0x2F700DCD, 0x176CC5C9,
"Turok - Dinosaur Hunter (U) (V1.1)/(v1.2) [!]"},
{0x66E4FA0F, 0xDE88C7D0,
"Turok - Legenden des Verlorenen Landes (Rage Wars) (G) [!]"},
{0x1EA26214, 0xE790900F,
"Turok - Rage Wars (E) [!]"},
{0x2E0E7749, 0xB8B49D59,
"Turok 2 - Seeds of Evil (FIS)"},
{0xADB9498B, 0xDAF28F55,
"Turok - Rage Wars (U) [!]"},
{0xE8C95AFC, 0x35D121DA,
"Turok 2 - Seeds of Evil - Kiosk (E) [!]"},
{0xE8C95AFC, 0x35D121DA,
"Turok 2 - Seeds of Evil - Kiosk (U) [!]"},
{0xE0B92B94, 0x80E87CBD,
"Turok 2 - Seeds of Evil (E) [!]"},
{0xFE05840B, 0x9393320C,
"Turok 2 - Seeds of Evil (G) [!]"},
{0x2E0E7749, 0xB8B49D59,
"Turok 2 - Seeds of Evil (FIS)"},
{0x49088A11, 0x6494957E,
"Turok 2 - Seeds of Evil (U) [!]"},
{0x6A162FF2, 0x2093704C,
"Turok 3 - Shadow of Oblivion (E) [!]"},
{0x89A579F1, 0x667E97EF,
"Turok 3 - Shadow of Oblivion (U) [!]"},
{0xE688A5B8, 0xB14B3F18,
"Twisted Edge Extreme Snowboarding (E) [!]"},
{0xBBC99D32, 0x117DAA80,
"Twisted Edge Extreme Snowboarding (U) [!]"},
{0xDD10BC7E, 0xF900B351,
"Vigilante 8 - 2nd Offence (E) [!]"},
{0xF5C5866D, 0x052713D9,
"Vigilante 8 - 2nd Offense (U) [!]"},
{0x151F79F4, 0x8EEDC8E5,
"Vigilante 8 (E) [!]"},
{0xE2BC82A2, 0x591CD694,
"Vigilante 8 (F) [!]"},
{0x6EDA5178, 0xD396FEC1,
"Vigilante 8 (G) [!]"},
{0xEA71056A, 0xE4214847,
"Vigilante 8 (U) [!]"},
{0x60006C98, 0x2605A381,
"Violence Killer - Turok New Generation (J) [!]"},
{0x2FDAA221, 0xA588A7CE,
"Virtual Chess 64 (E) [!]"},
{0x82B3248B, 0xE73E244D,
"Virtual Chess 64 (U) [!]"},
{0x98F9F2D0, 0x03D9F09C,
"Virtual Pool 64 (E) [!]"},
{0x4E4A7643, 0xA37439D7,
"Virtual Pool 64 (U) [!]"},
{0x045C08C4, 0x4AFD798B,
"Virtual Pro Wrestling (J) [!]"},
{0xCD094235, 0x88074B62,
"Virtual Pro Wrestling 2 (J) [!]"},
{0x636E6B19, 0xE57DDC5F,
"V-Rally Edition 99 (E) [!]"},
{0x4D0224A5, 0x1BEB5794,
"V-Rally Edition 99 (J) [!]"},
{0x3C059038, 0xC8BF2182,
"V-Rally Edition 99 (U) [!]"},
{0x93053075, 0x261E0F43,
"Waialae Country Club-True Golf Classics (E) (V1.0) [!]"},
{0x0C5057AD, 0x046E126E,
"Waialae Country Club-True Golf Classics (E) (V1.1) [!]"},
{0x8066D58A, 0xC3DECAC1,
"Waialae Country Club-True Golf Classics (U) [!]"},
{0xD715CC70, 0x271CF5D6,
"War Gods (E) [!]"},
{0xF7FE28F6, 0xC3F2ACC3,
"War Gods (U) [!]"},
{0x650EFA96, 0x30DDF9A7,
"Wave Race 64 (E) [!]"},
{0x5C9191D6, 0xB30AC306,
"Wave Race 64 (J) [!]"},
{0x44995484, 0x20A5FC5E,
"Wave Race 64 (J) (V1.1) [!]"},
{0x7DE11F53, 0x74872F9D,
"Wave Race 64 (U) (V1.0) [!]"},
{0x492F4B61, 0x04E5146A,
"Wave Race 64 (U) (V1.1) [!]"},
{0x535DF3E2, 0x609789F1,
"Wave Race 64 Shindou Edition (J) (V1.2) [!]"},
{0x2209094B, 0x2C9559AF,
"Wayne Gretzky's 3D Hockey (E) [!]"},
{0xF1301043, 0xFD80541A,
"Wayne Gretzky's 3D Hockey (J) [!]"},
{0x6B45223F, 0xF00E5C56,
"Wayne Gretzky's 3D Hockey (U) [!]"},
{0xDC3BAA59, 0x0ABB456A,
"Wayne Gretzky's 3D Hockey (U) (V1.1)"},
{0x661B45F3, 0x9ED6266D,
"Wayne Gretzky's 3D Hockey '98 (E) [!]"},
{0x5A9D3859, 0x97AAE710,
"Wayne Gretzky's 3D Hockey '98 (U) [!]"},
{0x396F5ADD, 0x6693ECA7,
"WCW Backstage Assault (U) [!]"},
{0xAA7B0658, 0x9C96937B,
"WCW Mayhem (E) [!]"},
{0x33BE8CD6, 0xEC186912,
"WCW Mayhem (U) [!]"},
{0xD4C45A1A, 0xF425B25E,
"WCW Nitro (U) [!]"},
{0x8BDBAF68, 0x345B4B36,
"WCW vs. nWo - World Tour (E) [!]"},
{0x2C3E19BD, 0x5113EE5E,
"WCW vs. nWo - World Tour (U) (V1.0) [!]"},
{0x71BE60B9, 0x1DDBFB3C,
"WCW vs. nWo - World Tour (U) (V1.1) [!]"},
{0x68E8A875, 0x0CE7A486,
"WCW-nWo Revenge (E) [!]"},
{0xDEE596AB, 0xAF3B7AE7,
"WCW-nWo Revenge (U) [!]"},
{0xCEA8B54F, 0x7F21D503,
"Wetrix (E) [!]"},
{0xDCB6EAFA, 0xC6BBCFA3,
"Wetrix (J) [!]"},
{0xCEA8B54F, 0x7F21D503,
"Wetrix (U) [!]"},
{0xE896092B, 0xDC244D4E,
"Wheel of Fortune (U) [!]"},
{0x0CEBC4C7, 0x0C9CE932,
"Wild Choppers (J) [!]"},
{0xD5898CAF, 0x6007B65B,
"WinBack - Covert Operations (E) [!]"},
{0x1FA056E0, 0xA4B9946A,
"WinBack - Covert Operations (J) [!]"},
{0xED98957E, 0x8242DCAC,
"WinBack - Covert Operations (U) [!]"},
{0x54310E7D, 0x6B5430D8,
"Wipeout 64 (E) [!]"},
{0x132D2732, 0xC70E9118,
"Wipeout 64 (U) [!]"},
{0xE43C9765, 0x05B1C1BE,
"Wonder Project J2 (J) [!]"},
{0xF9FC3090, 0xFF014EC2,
"World Cup 98 (E) [!]"},
{0xBD636D6A, 0x5D1F54BA,
"World Cup 98 (U) [!]"},
{0xAC062778, 0xDFADFCB8,
"World Driver Championship (E) [!]"},
{0x308DFEC8, 0xCE2EB5F6,
"World Driver Championship (U) [!]"},
{0x3B941695, 0xF90A5EEB,
"World is Not Enough, The (E) [!]"},
{0x033F4C13, 0x319EE7A7,
"World is Not Enough, The (U) [!]"},
{0x2D21C57B, 0x8FE4C58C,
"Worms - Armageddon (E) [!]"},
{0x13E959A0, 0x0E93CAB0,
"Worms - Armageddon (U) [!]"},
{0x33A275A4, 0xB8504459,
"WWF - War Zone (E) [!]"},
{0xCD5BEC0F, 0x86FD1008,
"WWF - War Zone (U) [!]"},
{0x5BF45B7B, 0x596BEEE8,
"WWF Attitude (E) [!]"},
{0x8F3151C8, 0x4F3AF545,
"WWF Attitude (G) [!]"},
{0xD2BE2F14, 0x38453788,
"WWF Attitude (U) [!]"},
{0x6D8DF08E, 0xD008C3CF,
"WWF No Mercy (E) (V1.0) [!]"},
{0x8CDB94C2, 0xCB46C6F0,
"WWF No Mercy (E) (V1.1) [!]"},
{0x4E4B0640, 0x1B49BCFB,
"WWF No Mercy (U) [!]"},
{0xC71353BE, 0xAA09A6EE,
"WWF WrestleMania 2000 (E) [!]"},
{0x12737DA5, 0x23969159,
"WWF Wrestlemania 2000 (J) [!]"},
{0x90A59003, 0x31089864,
"WWF WrestleMania 2000 (U) [!]"},
{0x0A1667C7, 0x293346A6,
"Xena Warrior Princess - Talisman of Fate (E) [!]"},
{0x0553AE9D, 0xEAD8E0C1,
"Xena Warrior Princess - Talisman of Fate (U) [!]"},
{0x9F8B96C3, 0xA01194DC,
"Yakouchuu II - Satsujun Kouru (J) [!]"},
{0xD3F97D49, 0x6924135B,
"Yoshi's Story (E) [!]"},
{0x2DCFCA60, 0x8354B147,
"Yoshi's Story (J) [!]"},
{0x2337D8E8, 0x6B8E7CEC,
"Yoshi's Story (U) [!]"},
{0x77DA3B8D, 0x162B0D7C,
" Yousuke Ide's Mahjong School (J) [!]"},
{0x9FE6162D, 0xE97E4037,
"Yuke Yuke!! Trouble Makers (J) [!]"},
{0xEC7011B7, 0x7616D72B,
"Zelda no Densetsu - Toki no Ocarina (J) (V1.0)"},
{0xD43DA81F, 0x021E1E19,
"Zelda no Densetsu - Toki no Ocarina (J) (V1.1)"},
{0x693BA2AE, 0xB7F14E9F,
"Zelda no Densetsu - Toki no Ocarina (J) (V1.2)"},
{0xF7F52DB8, 0x2195E636,
"Zelda no Densetsu - Toki no Ocarina - Zelda Collection Version (J) (GC Version)"},
{0xF43B45BA, 0x2F0E9B6F,
"Zelda no Densetsu - Toki no Ocarina GC Ura (J) (GC Version)"},
{0xF611F4BA, 0xC584135C,
"Zelda no Densetsu - Toki no Ocarina GC (J) (GC Version)"},
{0xEC417312, 0xEB31DE5F,
"Zelda no Densetsu - Mujura no Kamen (J) (V1.0)"},
{0x69AE0438, 0x2C63F3F3,
"Zelda no Densetsu - Mujura no Kamen (J) (V1.1)"},
{0x8473D0C1, 0x23120666,
"Zelda no Densetsu - Mujura no Kamen - Zelda Collection Version (J) (GC Version)"},
{0x1C010CCD, 0x22D3B7FA,
"Zool - Majou Tsukai Densetsu (J) [!]"},
};

#ifdef __cplusplus
}
#endif

#endif
